@@ -401,7 +401,7 @@ impl<'a> Ctx<'a> {
         let name = module.name()?.as_name();
         let visibility = self.lower_visibility(module);
         let kind = if module.semicolon_token().is_some() {
-            ModKind::Outline {}
+            ModKind::Outline
         } else {
             ModKind::Inline {
                 items: module
@@ -582,7 +582,7 @@ impl<'a> Ctx<'a> {
             }
             GenericsOwner::Trait(trait_def) => {
                 // traits get the Self type as an implicit first type parameter
-                generics.types.alloc(
+                generics.type_or_consts.alloc(
                     TypeParamData {
                         name: Some(name![Self]),
                         default: None,
